-- zchar.e
-- ZIG ASCII-set modification routines
-- Most routines written by David Cuny
-- 'load_rom_font' routine written by Jiri Babor
-- Some routines written by or modified by Darren Hewer (CornDog)


include machine.e
include graphics.e
include get.e

global sequence default_charset          

global function rtrim(sequence s)
    integer i
    i=length(s)
    while i do
        if s[i] then exit end if
        i=i-1
    end while
    return s[1..i]
end function

global function load_rom_font()
    sequence b,cb,cbm,regs,char
    integer a, val, total, cnt  -- val, total added
    regs = repeat(0,10)
    regs[REG_AX]=#1130
    regs[REG_BX]=#600
    regs=dos_interrupt(#10,regs)
    a=#10*regs[REG_ES]+regs[REG_BP]+16  -- bypass zero char
    char={}
    default_charset = repeat ({}, 255)
    for i=1 to 255 do                   -- again bypass zero        
        cbm={}
        cb=rtrim(peek({a,16}))          -- char bytes, right trimmed
        for j=1 to length(cb) do
            b=and_bits(cb[j],{128,64,32,16,8,4,2,1}) and 1
            cbm=append(cbm,rtrim(b))
        end for
        a=a+16
        char=append(char,cbm)         
        -- Darren's {crappy} code starts ^_^
        -- This stuff transforms the 0/1 vals to one 0-255 val
        -- For each line of the char    
        cnt = 16 - length(char[i])
        for q = 1 to length (char[i]) do  
            val = 256                  
            total = 0               
            -- For each 0/1 val of each line
            for z = 1 to length (char[i][q]) do
                val = val / 2
                if char[i][q][z] = 1 then
                    total = total + val        
                end if                      
            end for       
            default_charset[i] = default_charset[i] & total
        end for         
        -- Add extra 0's
        for p = 1 to cnt do
            default_charset[i] = default_charset[i] & 0    
        end for
        -- Darren's code ends ^_^
    end for                      
    -- Add character '0' onto the beginning
    default_charset = prepend (default_charset, {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0})
    return 0
end function
                            
----------------------------------------------------------------------------
-- Write a char to memory (edit the char)
global procedure add_font( integer char, sequence pattern )
    object    lowMem
    sequence  inReg, outReg
    lowMem = allocate_low( length( pattern ) )   -- get memory
    poke( lowMem, pattern )         -- move description to memory
    inReg = repeat( 0, 10 )         -- initialize register list
    inReg[REG_AX] = #1100           -- new char function
    inReg[REG_BX] = 16 * 256        -- 16 bytes/char - for 256 characters
    inReg[REG_CX] = 1               -- number of characters to change
    inReg[REG_DX] = char            -- first character to change
    inReg[REG_ES] = floor(lowMem / 16)    -- segment of low_mem (low memory)
    inReg[REG_BP] = remainder(lowMem, 16) -- offset of low_mem
    outReg = dos_interrupt( #10, inReg )  -- bios call
    free_low( lowMem )  -- free the memory
end procedure

global procedure add_all_fonts(integer offset, sequence pat2 )
    object    lowMem
    sequence  inReg, outReg, pattern
    pattern = {}
    -- there's probably a more efficient way to do this \/
    for z = 1 to length(pat2) do
        for a = 1 to 16 do
            pattern = append(pattern, pat2[z][a])
        end for
    end for
    lowMem = allocate_low( length( pattern ) )   -- get memory
    poke( lowMem, pattern )         -- move all fonts descriptions to memory
    inReg = repeat( 0, 10 )         -- initialize register list
    inReg[REG_AX] = #1100           -- new char function
    inReg[REG_BX] = 16 * 256        -- 16 bytes/char - for 256 characters
    inReg[REG_CX] = length(pat2)             -- number of characters to change
    inReg[REG_DX] = offset               -- first character to change
    inReg[REG_ES] = floor(lowMem / 16)    -- segment of low_mem (low memory)
    inReg[REG_BP] = remainder(lowMem, 16) -- offset of low_mem
    outReg = dos_interrupt( #10, inReg )  -- bios call
    free_low( lowMem )  -- free the memory
end procedure

global function pattern_to_binary(sequence pattern)
    integer accum    
    -- replace pattern with bits
    for i = 1 to 16 do        
        -- convert to binary
        accum = 0
        if pattern[i][1] = 'x' then accum = accum + 128       end if
        if pattern[i][2] = 'x' then accum = accum + 64        end if
        if pattern[i][3] = 'x' then accum = accum + 32        end if
        if pattern[i][4] = 'x' then accum = accum + 16        end if
        if pattern[i][5] = 'x' then accum = accum + 8         end if
        if pattern[i][6] = 'x' then accum = accum + 4         end if
        if pattern[i][7] = 'x' then accum = accum + 2         end if
        if pattern[i][8] = 'x' then accum = accum + 1         end if        
        -- replace with number
        pattern[i] = accum        
    end for
    return pattern
end function

global procedure font( integer char, sequence pattern )
    add_font( char, pattern_to_binary(pattern) )
end procedure

global procedure ZIGLoadFont(sequence fn)
    atom f
        if not atom(dir(fn)) then
            f = open(fn, "rb")
            CharSet = repeat(repeat(0, 16), 256)
            for z = 1 to 256 do
                for a = 1 to 16 do
                    CharSet[z][a] = getc(f)
                end for
            end for
            add_all_fonts(0, CharSet)
            close(f)    
        else
            CharSet = {}
            return
        end if
end procedure
                       
atom j
j = load_rom_font()

